@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2016-2021 Canonical, Ltd.
- * Copyright (C) 2021-2023 Colin Ian King.
+ * Copyright (C) 2021-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,13 +18,19 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
 #include "core-cpu-cache.h"
+#include "core-madvise.h"
 #include "core-nt-store.h"
+#include "core-out-of-memory.h"
 #include "core-target-clones.h"
 #include "core-vecmath.h"
 
-#define MR_RD			(0)
-#define MR_WR			(1)
+#include <time.h>
+
+#define MR_RD			(0x0001)
+#define MR_WR			(0x0002)
+#define MR_RW			(MR_RD | MR_WR)
 
 #define MIN_MEMRATE_BYTES       (4 * KB)
 #define MAX_MEMRATE_BYTES       (MAX_MEM_LIMIT)
@@ -36,10 +42,11 @@
 static const stress_help_t help[] = {
 	{ NULL,	"memrate N",		"start N workers exercised memory read/writes" },
 	{ NULL,	"memrate-bytes N",	"size of memory buffer being exercised" },
+	{ NULL,	"memrate-flush",	"flush cache before each iteration" },
+	{ NULL, "memrate-method M",	"specify read/write memory exercising method" },
 	{ NULL,	"memrate-ops N",	"stop after N memrate bogo operations" },
 	{ NULL,	"memrate-rd-mbs N",	"read rate from buffer in megabytes per second" },
 	{ NULL,	"memrate-wr-mbs N",	"write rate to buffer in megabytes per second" },
-	{ NULL,	"memrate-flush",	"flush cache before each iteration" },
 	{ NULL,	NULL,			NULL }
 };
 
@@ -63,6 +70,7 @@ typedef struct {
 	uint64_t memrate_bytes;
 	uint64_t memrate_rd_mbs;
 	uint64_t memrate_wr_mbs;
+	size_t memrate_method;
 	void *start;
 	void *end;
 	bool memrate_flush;
@@ -73,58 +81,23 @@ typedef uint64_t (*stress_memrate_func_t)(const stress_memrate_context_t *contex
 typedef struct {
 	const char 	*name;
 	const int	rdwr;
-	stress_memrate_func_t	func;
-	stress_memrate_func_t	func_rate;
+	const stress_memrate_func_t	func;
+	const stress_memrate_func_t	func_rate;
 } stress_memrate_info_t;
 
-static int stress_set_memrate_bytes(const char *opt)
-{
-	uint64_t memrate_bytes;
-
-	memrate_bytes = stress_get_uint64_byte(opt);
-	stress_check_range_bytes("memrate-bytes", memrate_bytes,
-		MIN_MEMRATE_BYTES, MAX_MEMRATE_BYTES);
-	return stress_set_setting("memrate-bytes", TYPE_ID_UINT64, &memrate_bytes);
-}
-
-static int stress_set_memrate_rd_mbs(const char *opt)
-{
-	uint64_t memrate_rd_mbs;
-
-	memrate_rd_mbs = stress_get_uint64(opt);
-	stress_check_range_bytes("memrate-rd-mbs", memrate_rd_mbs,
-		1, 1000000);
-	return stress_set_setting("memrate-rd-mbs", TYPE_ID_UINT64, &memrate_rd_mbs);
-}
-
-static int stress_set_memrate_wr_mbs(const char *opt)
-{
-	uint64_t memrate_wr_mbs;
-
-	memrate_wr_mbs = stress_get_uint64(opt);
-	stress_check_range_bytes("memrate-wr-mbs", memrate_wr_mbs,
-		1, 1000000);
-	return stress_set_setting("memrate-wr-mbs", TYPE_ID_UINT64, &memrate_wr_mbs);
-}
-
 static void NORETURN MLOCKED_TEXT stress_memrate_alarm_handler(int signum)
 {
         (void)signum;
         siglongjmp(jmpbuf, 1);
 }
 
-static int stress_set_memrate_flush(const char *opt)
-{
-	return stress_set_setting_true("memrate-flush", opt);
-}
-
 static uint64_t stress_memrate_loops(
 	const stress_memrate_context_t *context,
 	const size_t size)
 {
 	uint64_t chunk_shift = 20;	/* 1 MB */
-	uint64_t bytes = context->memrate_bytes;
-	uint64_t best_fit = bytes / size;
+	const uint64_t bytes = context->memrate_bytes;
+	const uint64_t best_fit = bytes / size;
 
 	/* check for powers of 2 size, from 1MB down to 1K */
 	for (chunk_shift = 20; chunk_shift >= 10; chunk_shift--) {
@@ -142,7 +115,7 @@ static uint64_t stress_memrate_loops(
 static void OPTIMIZE3 stress_memrate_flush(const stress_memrate_context_t *context)
 {
 	uint8_t *start ALIGNED(4096) = (uint8_t *)context->start;
-	uint8_t *end ALIGNED(4096) = (uint8_t *)context->end;
+	const uint8_t *end ALIGNED(4096) = (uint8_t *)context->end;
 
 	while (start < end) {
 		shim_clflush(start);
@@ -220,7 +193,7 @@ static uint64_t TARGET_CLONES OPTIMIZE3 stress_memrate_read_rate##size(		\
 	for (ptr = start; ptr < end;) {				\
 		double t2, dur_remainder;			\
 		const type *loop_end = ptr + loop_elements;	\
-		register type *read_end = (type *)		\
+		register const type *read_end = (type *)	\
 			STRESS_PTR_MINIMUM(loop_end, end);	\
 								\
 		while (ptr < read_end) {			\
@@ -268,7 +241,7 @@ static uint64_t TARGET_CLONES OPTIMIZE3 stress_memrate_read_rate##size(		\
 			time_t sec = (time_t)dur_remainder;	\
 								\
 			t.tv_sec = sec;				\
-			t.tv_nsec = (long)((dur_remainder -	\
+			t.tv_nsec = (long int)((dur_remainder -	\
 				(double)sec) *			\
 				STRESS_NANOSECOND);		\
 			(void)nanosleep(&t, NULL);		\
@@ -319,7 +292,7 @@ static uint64_t stress_memrate_memset(
 {
 	const size_t size = context->memrate_bytes;
 
-	(void)memset(context->start, 0xaa, size);
+	(void)shim_memset(context->start, 0xaa, size);
 
 	*valid = true;
 	return (uint64_t)size / KB;
@@ -339,7 +312,7 @@ static uint64_t OPTIMIZE3 stress_memrate_memset_rate(
 
 	t1 = stress_time_now();
 	for (ptr = start; (ptr + chunk_size) < end; ptr += chunk_size) {
-		(void)memset(ptr, 0xaa, chunk_size);
+		(void)shim_memset(ptr, 0xaa, chunk_size);
 
 		t2 = stress_time_now();
 		total_dur += dur;
@@ -350,7 +323,7 @@ static uint64_t OPTIMIZE3 stress_memrate_memset_rate(
 			time_t sec = (time_t)dur_remainder;
 
 			t.tv_sec = sec;
-			t.tv_nsec = (long)((dur_remainder -
+			t.tv_nsec = (long int)((dur_remainder -
 				(double)sec) *
 				STRESS_NANOSECOND);
 			(void)nanosleep(&t, NULL);
@@ -358,7 +331,7 @@ static uint64_t OPTIMIZE3 stress_memrate_memset_rate(
 	}
 
 	if (end - ptr > 0) {
-		(void)memset(ptr, 0xaa, end - ptr);
+		(void)shim_memset(ptr, 0xaa, end - ptr);
 		t2 = stress_time_now();
 		total_dur += dur;
 		dur_remainder = total_dur - (t2 - t1);
@@ -368,7 +341,7 @@ static uint64_t OPTIMIZE3 stress_memrate_memset_rate(
 			time_t sec = (time_t)dur_remainder;
 
 			t.tv_sec = sec;
-			t.tv_nsec = (long)((dur_remainder -
+			t.tv_nsec = (long int)((dur_remainder -
 				(double)sec) *
 				STRESS_NANOSECOND);
 			(void)nanosleep(&t, NULL);
@@ -392,7 +365,7 @@ static uint64_t TARGET_CLONES OPTIMIZE3	stress_memrate_write##size(	\
 	{							\
 		type vaa;					\
 								\
-		(void)memset(&vaa, 0xaa, sizeof(vaa));		\
+		(void)shim_memset(&vaa, 0xaa, sizeof(vaa));	\
 		v = vaa;					\
 	}							\
 								\
@@ -478,7 +451,7 @@ static inline uint64_t OPTIMIZE3 stress_memrate_stos_rate(
 			time_t sec = (time_t)dur_remainder;
 
 			t.tv_sec = sec;
-			t.tv_nsec = (long)((dur_remainder -
+			t.tv_nsec = (long int)((dur_remainder -
 				(double)sec) *
 				STRESS_NANOSECOND);
 			(void)nanosleep(&t, NULL);
@@ -498,7 +471,7 @@ static inline uint64_t OPTIMIZE3 stress_memrate_stos_rate(
 			time_t sec = (time_t)dur_remainder;
 
 			t.tv_sec = sec;
-			t.tv_nsec = (long)((dur_remainder -
+			t.tv_nsec = (long int)((dur_remainder -
 				(double)sec) *
 				STRESS_NANOSECOND);
 			(void)nanosleep(&t, NULL);
@@ -548,7 +521,7 @@ static inline uint64_t OPTIMIZE3 stress_memrate_write_stos_rate64(
     !defined(__ILP32__)
 static inline void OPTIMIZE3 stress_memrate_stosd(void *ptr, const uint32_t loops)
 {
-	register void *p = ptr;
+	register const void *p = ptr;
 	register const uint32_t l = loops;
 
 	__asm__ __volatile__(
@@ -581,7 +554,7 @@ static inline uint64_t OPTIMIZE3 stress_memrate_write_stos_rate32(
     !defined(__ILP32__)
 static inline void OPTIMIZE3 stress_memrate_stosw(void *ptr, const uint32_t loops)
 {
-	register void *p = ptr;
+	register const void *p = ptr;
 	register const uint32_t l = loops;
 
 	__asm__ __volatile__(
@@ -614,7 +587,7 @@ static inline uint64_t OPTIMIZE3 stress_memrate_write_stos_rate16(
     !defined(__ILP32__)
 static inline void OPTIMIZE3 stress_memrate_stosb(void *ptr, const uint32_t loops)
 {
-	register void *p = ptr;
+	register const void *p = ptr;
 	register const uint32_t l = loops;
 
 	__asm__ __volatile__(
@@ -662,7 +635,7 @@ static uint64_t TARGET_CLONES OPTIMIZE3 stress_memrate_write_rate##size(	\
 	{							\
 		type vaa;					\
 								\
-		(void)memset(&vaa, 0xaa, sizeof(vaa));		\
+		(void)shim_memset(&vaa, 0xaa, sizeof(vaa));	\
 		v = vaa;					\
 	}							\
 								\
@@ -670,7 +643,7 @@ static uint64_t TARGET_CLONES OPTIMIZE3 stress_memrate_write_rate##size(	\
 	for (ptr = start; ptr < end;) {				\
 		double t2, dur_remainder;			\
 		const type *loop_end = ptr + loop_elements;	\
-		register type *write_end = (type *)		\
+		register const type *write_end = (type *)	\
 			STRESS_PTR_MINIMUM(loop_end, end);	\
 								\
 		while (ptr < write_end) {			\
@@ -701,7 +674,7 @@ static uint64_t TARGET_CLONES OPTIMIZE3 stress_memrate_write_rate##size(	\
 			time_t sec = (time_t)dur_remainder;	\
 								\
 			t.tv_sec = sec;				\
-			t.tv_nsec = (long)((dur_remainder -	\
+			t.tv_nsec = (long int)((dur_remainder -	\
 				(double)sec) * 			\
 				STRESS_NANOSECOND);		\
 			(void)nanosleep(&t, NULL);		\
@@ -717,7 +690,7 @@ static uint64_t TARGET_CLONES OPTIMIZE3 stress_memrate_write_rate##size(	\
  *  non-temporal writes using movntdq. Data is not going to be
  *  read, so no need to cache. Write directly to memory.
  */
-#define STRESS_MEMRATE_WRITE_NT(size, type, op, init)		\
+#define STRESS_MEMRATE_WRITE_NT(size, type, op)			\
 static uint64_t OPTIMIZE3 stress_memrate_write_nt##size(	\
 	const stress_memrate_context_t *context,		\
 	bool *valid)						\
@@ -734,7 +707,7 @@ static uint64_t OPTIMIZE3 stress_memrate_write_nt##size(	\
 	{							\
 		type vaa;					\
 								\
-		(void)memset(&vaa, 0xaa, sizeof(vaa));		\
+		(void)shim_memset(&vaa, 0xaa, sizeof(vaa));	\
 		v = vaa;					\
 	}							\
 								\
@@ -763,7 +736,7 @@ static uint64_t OPTIMIZE3 stress_memrate_write_nt##size(	\
 	return ((uintptr_t)ptr - (uintptr_t)start) / KB;	\
 }
 
-#define STRESS_MEMRATE_WRITE_NT_RATE(size, type, op, init)	\
+#define STRESS_MEMRATE_WRITE_NT_RATE(size, type, op)		\
 static uint64_t OPTIMIZE3 stress_memrate_write_nt_rate##size(	\
 	const stress_memrate_context_t *context,		\
 	bool *valid)						\
@@ -787,7 +760,7 @@ static uint64_t OPTIMIZE3 stress_memrate_write_nt_rate##size(	\
 	{							\
 		type vaa;					\
 								\
-		(void)memset(&vaa, 0xaa, sizeof(vaa));		\
+		(void)shim_memset(&vaa, 0xaa, sizeof(vaa));	\
 		v = vaa;					\
 	}							\
 								\
@@ -795,7 +768,7 @@ static uint64_t OPTIMIZE3 stress_memrate_write_nt_rate##size(	\
 	for (ptr = start; ptr < end;) {				\
 		double t2, dur_remainder;			\
 		const type *loop_end = ptr + loop_elements;	\
-		register type *write_end = (type *)		\
+		register const type *write_end = (type *)	\
 			STRESS_PTR_MINIMUM(loop_end, end);	\
 								\
 		while (ptr < write_end) {			\
@@ -828,7 +801,7 @@ static uint64_t OPTIMIZE3 stress_memrate_write_nt_rate##size(	\
 			time_t sec = (time_t)dur_remainder;	\
 								\
 			t.tv_sec = sec;				\
-			t.tv_nsec = (long)((dur_remainder -	\
+			t.tv_nsec = (long int)((dur_remainder -	\
 				(double)sec) * 			\
 				STRESS_NANOSECOND);		\
 			(void)nanosleep(&t, NULL);		\
@@ -839,18 +812,18 @@ static uint64_t OPTIMIZE3 stress_memrate_write_nt_rate##size(	\
 }
 
 #if defined(HAVE_NT_STORE128)
-STRESS_MEMRATE_WRITE_NT(128, __uint128_t, stress_nt_store128, i)
-STRESS_MEMRATE_WRITE_NT_RATE(128, __uint128_t, stress_nt_store128, i)
+STRESS_MEMRATE_WRITE_NT(128, __uint128_t, stress_nt_store128)
+STRESS_MEMRATE_WRITE_NT_RATE(128, __uint128_t, stress_nt_store128)
 #endif
 
 #if defined(HAVE_NT_STORE64)
-STRESS_MEMRATE_WRITE_NT(64, uint64_t, stress_nt_store64, i)
-STRESS_MEMRATE_WRITE_NT_RATE(64, uint64_t, stress_nt_store64, i)
+STRESS_MEMRATE_WRITE_NT(64, uint64_t, stress_nt_store64)
+STRESS_MEMRATE_WRITE_NT_RATE(64, uint64_t, stress_nt_store64)
 #endif
 
 #if defined(HAVE_NT_STORE32)
-STRESS_MEMRATE_WRITE_NT(32, uint32_t, stress_nt_store32, i)
-STRESS_MEMRATE_WRITE_NT_RATE(32, uint32_t, stress_nt_store32, i)
+STRESS_MEMRATE_WRITE_NT(32, uint32_t, stress_nt_store32)
+STRESS_MEMRATE_WRITE_NT_RATE(32, uint32_t, stress_nt_store32)
 #endif
 
 #if defined(HAVE_VECMATH)
@@ -877,6 +850,7 @@ STRESS_MEMRATE_WRITE(8, uint8_t)
 STRESS_MEMRATE_WRITE_RATE(8, uint8_t)
 
 static stress_memrate_info_t memrate_info[] = {
+	{ "all",	MR_RW,  NULL,				NULL },
 #if defined(HAVE_ASM_X86_REP_STOSQ) &&	\
     !defined(__ILP32__)
 	{ "write64stoq", MR_WR,	stress_memrate_write_stos64,	stress_memrate_write_stos_rate64 },
@@ -949,14 +923,11 @@ static void OPTIMIZE3 stress_memrate_init_data(
 		*ptr = stress_mwc32();
 }
 
-static inline void *stress_memrate_mmap(const stress_args_t *args, uint64_t sz)
+static inline void *stress_memrate_mmap(stress_args_t *args, uint64_t sz)
 {
 	void *ptr;
 
-	ptr = mmap(NULL, (size_t)sz, PROT_READ | PROT_WRITE,
-#if defined(MAP_POPULATE)
-		MAP_POPULATE |
-#endif
+	ptr = stress_mmap_populate(NULL, (size_t)sz, PROT_READ | PROT_WRITE,
 #if defined(HAVE_MADVISE)
 		MAP_PRIVATE |
 #else
@@ -974,6 +945,7 @@ static inline void *stress_memrate_mmap(const stress_args_t *args, uint64_t sz)
 
 		VOID_RET(int, madvise(ptr, sz, MADV_HUGEPAGE));
 #endif
+		(void)stress_madvise_mergeable(ptr, sz);
 	}
 	return ptr;
 }
@@ -983,22 +955,48 @@ static inline uint64_t stress_memrate_dispatch(
 	const stress_memrate_context_t *context,
 	bool *valid)
 {
-	if (((info->rdwr == MR_RD) && (context->memrate_rd_mbs == ~0ULL)) ||
-	    ((info->rdwr == MR_WR) && (context->memrate_wr_mbs == ~0ULL))) {
+	if (((info->rdwr == MR_RD) && (context->memrate_rd_mbs == 0ULL)) ||
+	    ((info->rdwr == MR_WR) && (context->memrate_wr_mbs == 0ULL))) {
+		return 0;
+	} else if (((info->rdwr == MR_RD) && (context->memrate_rd_mbs == ~0ULL)) ||
+		 ((info->rdwr == MR_WR) && (context->memrate_wr_mbs == ~0ULL))) {
 		return info->func(context, valid);
+	} else {
+		return info->func_rate(context, valid);
 	}
-	return info->func_rate(context, valid);
 }
 
-static int stress_memrate_child(const stress_args_t *args, void *ctxt)
+static void stress_memrate_dispatch_method(
+	const stress_memrate_context_t *context,
+	const size_t method)
+{
+	double t1, t2;
+	uint64_t kbytes;
+	const stress_memrate_info_t *info = &memrate_info[method];
+	bool valid = false;
+
+	if (context->memrate_flush)
+		stress_memrate_flush(context);
+	t1 = stress_time_now();
+	kbytes = stress_memrate_dispatch(info, context, &valid);
+	context->stats[method].kbytes += (double)kbytes;
+	t2 = stress_time_now();
+	context->stats[method].duration += (t2 - t1);
+	context->stats[method].valid = valid;
+}
+
+static int stress_memrate_child(stress_args_t *args, void *ctxt)
 {
 	stress_memrate_context_t *context = (stress_memrate_context_t *)ctxt;
 	void *buffer, *buffer_end;
 
+	stress_catch_sigill();
+
 	buffer = stress_memrate_mmap(args, context->memrate_bytes);
 	if (buffer == MAP_FAILED)
 		return EXIT_NO_RESOURCE;
 
+	stress_set_vma_anon_name(buffer, context->memrate_bytes, "memrate-buffer");
 	buffer_end = (uint8_t *)buffer + context->memrate_bytes;
 	stress_memrate_init_data(buffer, buffer_end);
 
@@ -1014,26 +1012,17 @@ static int stress_memrate_child(const stress_args_t *args, void *ctxt)
 	do {
 		size_t i;
 
-		for (i = 0; i < memrate_items; i++) {
-			double t1, t2;
-			uint64_t kbytes;
-			stress_memrate_info_t *info = &memrate_info[i];
-			bool valid = false;
-
-			if (context->memrate_flush)
-				stress_memrate_flush(context);
-			t1 = stress_time_now();
-			kbytes = stress_memrate_dispatch(info, context, &valid);
-			context->stats[i].kbytes += (double)kbytes;
-			t2 = stress_time_now();
-			context->stats[i].duration += (t2 - t1);
-			context->stats[i].valid = valid;
-
-			if (!keep_stressing(args))
-				break;
+		if (context->memrate_method == 0) {
+			for (i = 1; i < memrate_items; i++) {
+				stress_memrate_dispatch_method(context, i);
+				if (UNLIKELY(!stress_continue(args)))
+					break;
+			}
+		} else {
+			stress_memrate_dispatch_method(context, context->memrate_method);
 		}
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
 tidy:
 	(void)munmap((void *)buffer, context->memrate_bytes);
@@ -1044,7 +1033,7 @@ tidy:
  *  stress_memrate()
  *	stress cache/memory/CPU with memrate stressors
  */
-static int stress_memrate(const stress_args_t *args)
+static int stress_memrate(stress_args_t *args)
 {
 	int rc;
 	size_t i, stats_size;
@@ -1054,16 +1043,31 @@ static int stress_memrate(const stress_args_t *args)
 	context.memrate_rd_mbs = ~0ULL;
 	context.memrate_wr_mbs = ~0ULL;
 	context.memrate_flush = false;
+	context.memrate_method = 0; 	/* all */
+	int flag;
 
 	(void)stress_get_setting("memrate-bytes", &context.memrate_bytes);
 	(void)stress_get_setting("memrate-flush", &context.memrate_flush);
 	(void)stress_get_setting("memrate-rd-mbs", &context.memrate_rd_mbs);
 	(void)stress_get_setting("memrate-wr-mbs", &context.memrate_wr_mbs);
+	(void)stress_get_setting("memrate-method", &context.memrate_method);
+
+	if ((context.memrate_rd_mbs == 0ULL) && (context.memrate_wr_mbs == 0ULL)) {
+		pr_fail("%s: cannot use zero MB rates for read and write\n", args->name);
+		return EXIT_FAILURE;
+	}
+	flag = ((context.memrate_rd_mbs == 0) ? 0 : MR_RD) |
+	       ((context.memrate_wr_mbs == 0) ? 0 : MR_WR);
+	if ((flag & memrate_info[context.memrate_method].rdwr) == 0) {
+		pr_fail("%s: cannot use zero MB rate and just methood %s\n",
+			args->name, memrate_info[context.memrate_method].name);
+		return EXIT_FAILURE;
+	}
 
 	stats_size = memrate_items * sizeof(*context.stats);
 	stats_size = (stats_size + args->page_size - 1) & ~(args->page_size - 1);
 
-	context.stats = (stress_memrate_stats_t *)mmap(NULL, stats_size,
+	context.stats = (stress_memrate_stats_t *)stress_mmap_populate(NULL, stats_size,
 		PROT_READ | PROT_WRITE, MAP_SHARED | MAP_ANONYMOUS, -1, 0);
 	if (context.stats == MAP_FAILED) {
 		pr_inf_skip("%s: failed to mmap %zd byte statistics buffer, skipping stressor\n",
@@ -1082,20 +1086,22 @@ static int stress_memrate(const stress_args_t *args)
 			context.memrate_bytes >> 10,
 			context.memrate_flush ? "enabled" : "disabled");
 		if ((context.memrate_bytes > MB) && (context.memrate_bytes & MB)) {
-			pr_inf("%s: for optimial speed, use multiples of 1 MB for --memrate-bytes\n", args->name);
+			pr_inf("%s: for optimal speed, use multiples of 1 MB for --memrate-bytes\n", args->name);
 		}
 		if (!context.memrate_flush)
 			pr_inf("%s: cache flushing can be enabled with --memrate-flush option\n", args->name);
 	}
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	rc = stress_oomable_child(args, &context, stress_memrate_child, STRESS_OOMABLE_NORMAL);
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
-	pr_lock();
-	for (i = 0; i < memrate_items; i++) {
+	pr_block_begin();
+	for (i = 1; i < memrate_items; i++) {
 		if (!context.stats[i].valid)
 			continue;
 		if (context.stats[i].duration > 0.0) {
@@ -1103,30 +1109,38 @@ static int stress_memrate(const stress_args_t *args)
 			const double rate = context.stats[i].kbytes / (context.stats[i].duration * KB);
 
 			(void)snprintf(tmp, sizeof(tmp), "%s MB per sec", memrate_info[i].name);
-			stress_metrics_set(args, i, tmp, rate);
+			stress_metrics_set(args, i, tmp,
+				rate, STRESS_METRIC_HARMONIC_MEAN);
 		} else {
 			pr_inf("%s: %10.10s: interrupted early\n",
 				args->name, memrate_info[i].name);
 		}
 	}
-	pr_unlock();
+	pr_block_end();
 
 	(void)munmap((void *)context.stats, stats_size);
 
 	return rc;
 }
 
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_memrate_bytes,	stress_set_memrate_bytes },
-	{ OPT_memrate_flush,	stress_set_memrate_flush },
-	{ OPT_memrate_rd_mbs,	stress_set_memrate_rd_mbs },
-	{ OPT_memrate_wr_mbs,	stress_set_memrate_wr_mbs },
-	{ 0,			NULL }
+static const char *stress_memmap_method(const size_t i)
+{
+        return (i < SIZEOF_ARRAY(memrate_info)) ? memrate_info[i].name : NULL;
+}
+
+
+static const stress_opt_t opts[] = {
+	{ OPT_memrate_bytes,  "memrate-bytes",  TYPE_ID_UINT64_BYTES_VM, MIN_MEMRATE_BYTES, MAX_MEMRATE_BYTES, NULL },
+	{ OPT_memrate_flush,  "memrate-flush",  TYPE_ID_BOOL, 0, 1, NULL },
+	{ OPT_memrate_rd_mbs, "memrate-rd-mbs", TYPE_ID_UINT64, 0, 1000000, NULL },
+	{ OPT_memrate_wr_mbs, "memrate-wr-mbs", TYPE_ID_UINT64, 0, 1000000, NULL },
+	{ OPT_memrate_method, "memrate-method", TYPE_ID_SIZE_T_METHOD, 0, 0, stress_memmap_method },
+	END_OPT,
 };
 
-stressor_info_t stress_memrate_info = {
+const stressor_info_t stress_memrate_info = {
 	.stressor = stress_memrate,
 	.class = CLASS_MEMORY,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
 	.help = help
 };
